@@ -15,19 +15,13 @@
  *
  * You should have received a copy of the GNU Lesser General
  * Public License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
+ * Boston, MA  02110-1301  USA
  */
 #include <config.h>
 
-#include <assert.h>
-#include <memory.h>
-#include <complex>
-#if USE_JACK
-#include <jack/jack.h>
-#endif
 #include <calf/giface.h>
-#include <calf/modules_synths.h>
+#include <calf/organ.h>
 #include <iostream>
 
 using namespace std;
@@ -36,7 +30,50 @@ using namespace calf_plugins;
 
 //////////////////////////////////////////////////////////////////////////////////////////////////////////
 
-bool organ_audio_module::get_graph(int index, int subindex, float *data, int points, cairo_iface *context)
+organ_audio_module::organ_audio_module()
+: drawbar_organ(&par_values)
+{
+    var_map_curve = "2\n0 1\n1 1\n"; // XXXKF hacky bugfix
+}
+
+void organ_audio_module::activate()
+{
+    setup(srate);
+    panic_flag = false;
+}
+
+void organ_audio_module::post_instantiate()
+{
+    dsp::organ_voice_base::precalculate_waves(progress_report);
+}
+
+
+uint32_t organ_audio_module::process(uint32_t offset, uint32_t nsamples, uint32_t inputs_mask, uint32_t outputs_mask)
+{
+    float *o[2] = { outs[0] + offset, outs[1] + offset };
+    if (panic_flag)
+    {
+        control_change(120, 0); // stop all sounds
+        control_change(121, 0); // reset all controllers
+        panic_flag = false;
+    }
+    render_separate(o, nsamples);
+    return 3;
+}
+
+void organ_audio_module::params_changed() {
+    for (int i = 0; i < param_count; i++)
+        ((float *)&par_values)[i] = *params[i];
+
+    unsigned int old_poly = polyphony_limit;
+    polyphony_limit = dsp::clip(dsp::fastf2i_drm(*params[par_polyphony]), 1, 32);
+    if (polyphony_limit < old_poly)
+        trim_voices();
+    
+    update_params();
+}
+
+bool organ_audio_module::get_graph(int index, int subindex, float *data, int points, cairo_iface *context, int *mode) const
 {
     if (index == par_master) {
         organ_voice_base::precalculate_waves(progress_report);
@@ -75,6 +112,13 @@ bool organ_audio_module::get_graph(int index, int subindex, float *data, int poi
     return false;
 }
 
+uint32_t organ_audio_module::message_run(const void *valid_inputs, void *output_ports)
+{ 
+    // silence a default printf (which is kind of a warning about unhandled message_run)
+    return 0;
+}
+
+
 ////////////////////////////////////////////////////////////////////////////
 
 organ_voice_base::small_wave_family (*organ_voice_base::waves)[organ_voice_base::wave_count_small];
@@ -174,9 +218,10 @@ static void padsynth(bandlimiter<ORGAN_WAVE_BITS> blSrc, bandlimiter<ORGAN_BIG_W
     // same as above - put large array on heap to avoid stack overflow in ingen
     vector<float> tmp;
     tmp.resize(ORGAN_BIG_WAVE_SIZE);
-    blDest.compute_waveform(tmp.data());
-    normalize_waveform(tmp.data(), ORGAN_BIG_WAVE_SIZE);
-    blDest.compute_spectrum(tmp.data());
+    float *ptmp = &tmp.front();
+    blDest.compute_waveform(ptmp);
+    normalize_waveform(ptmp, ORGAN_BIG_WAVE_SIZE);
+    blDest.compute_spectrum(ptmp);
     
     // limit is 1/2 of the number of harmonics of the original wave
     result.make_from_spectrum(blDest, foldover, ORGAN_WAVE_SIZE >> (1 + ORGAN_BIG_WAVE_SHIFT));
@@ -192,6 +237,12 @@ static void padsynth(bandlimiter<ORGAN_WAVE_BITS> blSrc, bandlimiter<ORGAN_BIG_W
 
 #define LARGE_WAVEFORM_PROGRESS() do { if (reporter) { progress += 100; reporter->report_progress(floor(progress / totalwaves), "Precalculating large waveforms"); } } while(0)
 
+void organ_voice_base::update_pitch()
+{
+    float phase = dsp::midi_note_to_phase(note, 100 * parameters->global_transpose + parameters->global_detune, sample_rate_ref);
+    dpphase.set((long int) (phase * parameters->percussion_harmonic * parameters->pitch_bend));
+    moddphase.set((long int) (phase * parameters->percussion_fm_harmonic * parameters->pitch_bend));
+}
 
 void organ_voice_base::precalculate_waves(progress_report_iface *reporter)
 {
@@ -502,6 +553,17 @@ void organ_voice_base::render_percussion_to(float (*buf)[2], int nsamples)
     }
 }
 
+void organ_voice_base::perc_reset()
+{
+    pphase = 0;
+    modphase = 0;
+    dpphase = 0;
+    moddphase = 0;
+    note = -1;
+}
+
+//////////////////////////////////////////////////////////////////////////////////////////////////////
+
 void organ_vibrato::reset()
 {
     for (int i = 0; i < VibratoSize; i++)
@@ -549,6 +611,87 @@ void organ_vibrato::process(organ_parameters *parameters, float (*data)[2], unsi
     }
 }
 
+void scanner_vibrato::reset()
+{
+    legacy.reset();
+    for (int i = 0; i < ScannerSize; i++)
+        scanner[i].reset();
+    lfo_phase = 0.f;
+}
+
+void scanner_vibrato::process(organ_parameters *parameters, float (*data)[2], unsigned int len, float sample_rate)
+{
+    if (!len)
+        return;
+    
+    int vtype = (int)parameters->lfo_type;
+    if (!vtype || vtype > organ_enums::lfotype_cvfull)
+    {
+        legacy.process(parameters, data, len, sample_rate);
+        return;
+    }
+    
+    // I bet the original components of the line box had some tolerance,
+    // hence two different values of cutoff frequency
+    scanner[0].set_lp_rbj(4000, 0.707, sample_rate);
+    scanner[1].set_lp_rbj(4200, 0.707, sample_rate);
+    for (int t = 2; t < ScannerSize; t ++)
+    {
+        scanner[t].copy_coeffs(scanner[t & 1]);
+    }
+    
+    float lfo_phase2 = lfo_phase + parameters->lfo_phase * (1.0 / 360.0);
+    if (lfo_phase2 >= 1.0)
+        lfo_phase2 -= 1.0;
+    float vib_wet = parameters->lfo_wet;
+    float dphase = parameters->lfo_rate / sample_rate;
+    static const int v1[] = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 8, 8 };
+    static const int v2[] = { 0, 1, 2, 4, 6, 8, 9, 10, 12 };
+    static const int v3[] = { 0, 1, 3, 6, 11, 12, 15, 17, 18, 18, 18 };
+    static const int vfull[] = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 18 };
+    static const int *vtypes[] = { NULL, v1, v2, v3, vfull };
+    const int *vib = vtypes[vtype];
+    
+    float vibamt = 8 * parameters->lfo_amt;
+    if (vtype == organ_enums::lfotype_cvfull)
+        vibamt = 17 * parameters->lfo_amt;
+    for (unsigned int i = 0; i < len; i++)
+    {
+        float line[ScannerSize + 1];
+        float v0 = (data[i][0] + data[i][1]) * 0.5;
+        
+        line[0] = v0;
+        for (int t = 0; t < ScannerSize; t++)
+            line[t + 1] = scanner[t].process(line[t]) * 1.03;
+        
+        float lfo1 = lfo_phase < 0.5 ? 2 * lfo_phase : 2 - 2 * lfo_phase;
+        float lfo2 = lfo_phase2 < 0.5 ? 2 * lfo_phase2 : 2 - 2 * lfo_phase2;
+        
+        float pos = vibamt * lfo1;
+        int ipos = (int)pos;
+        float vl = lerp(line[vib[ipos]], line[vib[ipos + 1]], pos - ipos);
+        
+        pos = vibamt * lfo2;
+        ipos = (int)pos;
+        float vr = lerp(line[vib[ipos]], line[vib[ipos + 1]], pos - ipos);
+        
+        lfo_phase += dphase;
+        if (lfo_phase >= 1.0)
+            lfo_phase -= 1.0;
+        lfo_phase2 += dphase;
+        if (lfo_phase2 >= 1.0)
+            lfo_phase2 -= 1.0;
+        
+        data[i][0] += (vl - v0) * vib_wet;
+        data[i][1] += (vr - v0) * vib_wet;
+    }
+    for (int t = 0; t < ScannerSize; t++)
+    {
+        scanner[t].sanitize();
+    }
+}
+//////////////////////////////////////////////////////////////////////////////////////////////////////
+
 void organ_voice::update_pitch()
 {
     organ_voice_base::update_pitch();
@@ -661,7 +804,10 @@ void organ_voice::render_block() {
         }
         if (i) mod += expression.get() * 1200 * 4;
         float fc = parameters->filters[i].cutoff * pow(2.0f, mod * (1.f / 1200.f));
-        filterL[i].set_lp_rbj(dsp::clip<float>(fc, 10, 18000), parameters->filters[i].resonance, sample_rate);
+        if (i == 0 && parameters->filter1_type >= 0.5f)
+            filterL[i].set_hp_rbj(dsp::clip<float>(fc, 10, 18000), parameters->filters[i].resonance, sample_rate);
+        else
+            filterL[i].set_lp_rbj(dsp::clip<float>(fc, 10, 18000), parameters->filters[i].resonance, sample_rate);
         filterR[i].copy_coeffs(filterL[i]);
     }
     float amp_pre[ampctl_count - 1], amp_post[ampctl_count - 1];
@@ -727,8 +873,64 @@ void organ_voice::render_block() {
     
     if (use_percussion())
         render_percussion_to(output_buffer, BlockSize);
+
 }
 
+void organ_voice::note_on(int note, int vel)
+{
+    stolen = false;
+    finishing = false;
+    perc_released = false;
+    released = false;
+    reset();
+    this->note = note;
+    const float sf = 0.001f;
+    for (int i = 0; i < EnvCount; i++)
+    {
+        organ_parameters::organ_env_parameters &p = parameters->envs[i];
+        envs[i].set(sf * p.attack, sf * p.decay, p.sustain, sf * p.release, sample_rate / BlockSize);
+        envs[i].note_on();
+    }
+    update_pitch();
+    velocity = vel * 1.0 / 127.0;
+    amp.set(1.0f);
+    perc_note_on(note, vel);
+}
+
+void organ_voice::note_off(int /* vel */)
+{
+    // reset age to 0 (because decay will turn from exponential to linear, necessary because of error cumulation prevention)
+    perc_released = true;
+    if (pamp.get_active())
+    {
+        pamp.reinit();
+    }
+    rel_age_const = pamp.get() * ((1.0/44100.0)/0.03);
+    for (int i = 0; i < EnvCount; i++)
+        envs[i].note_off();
+}
+
+void organ_voice::steal()
+{
+    perc_released = true;
+    finishing = true;
+    stolen = true;
+}
+
+void organ_voice::reset()
+{
+    inertia_pitchbend.ramp.set_length(sample_rate / (BlockSize * 30)); // 1/30s    
+    vibrato.reset();
+    phase = 0;
+    for (int i = 0; i < FilterCount; i++)
+    {
+        filterL[i].reset();
+        filterR[i].reset();
+    }
+}
+
+///////////////////////////////////////////////////////////////////////////////////////////////////
+
 void drawbar_organ::update_params()
 {
     parameters->perc_decay_const = dsp::decay::calc_exp_constant(1.0 / 1024.0, 0.001 * parameters->percussion_time * sample_rate);
@@ -742,6 +944,42 @@ void drawbar_organ::update_params()
     parameters->foldvalue = (int)(dphase);
 }
 
+dsp::voice *drawbar_organ::alloc_voice()
+{
+    block_voice<organ_voice> *v = new block_voice<organ_voice>();
+    v->parameters = parameters;
+    return v;
+}
+
+void drawbar_organ::percussion_note_on(int note, int vel)
+{
+    percussion.perc_note_on(note, vel);
+}
+
+void drawbar_organ::setup(int sr)
+{
+    basic_synth::setup(sr);
+    percussion.setup(sr);
+    parameters->cutoff = 0;
+    params_changed();
+    global_vibrato.reset();
+}
+
+bool drawbar_organ::check_percussion() { 
+    switch(dsp::fastf2i_drm(parameters->percussion_trigger))
+    {        
+        case organ_voice_base::perctrig_first:
+            return active_voices.empty();
+        case organ_voice_base::perctrig_each: 
+        default:
+            return true;
+        case organ_voice_base::perctrig_eachplus:
+            return !percussion.get_noticable();
+        case organ_voice_base::perctrig_polyphonic:
+            return false;
+    }
+}
+
 void drawbar_organ::pitch_bend(int amt)
 {
     parameters->pitch_bend = pow(2.0, (amt * parameters->pitch_bend_range) / (1200.0 * 8192.0));
@@ -791,6 +1029,8 @@ char *organ_audio_module::configure(const char *key, const char *value)
 {
     if (!strcmp(key, "map_curve"))
     {
+        if (!value)
+            value = "2\n0 1\n1 1\n";
         var_map_curve = value;
         stringstream ss(value);
         int i = 0;
@@ -830,3 +1070,26 @@ void organ_audio_module::deactivate()
 {
     
 }
+
+void drawbar_organ::render_separate(float *output[], int nsamples)
+{
+    float buf[MAX_SAMPLE_RUN][2];
+    dsp::zero(&buf[0][0], 2 * nsamples);
+    basic_synth::render_to(buf, nsamples);
+    if (dsp::fastf2i_drm(parameters->lfo_mode) == organ_voice_base::lfomode_global)
+    {
+        for (int i = 0; i < nsamples; i += 64)
+            global_vibrato.process(parameters, buf + i, std::min(64, nsamples - i), sample_rate);
+    }
+    if (percussion.get_active())
+        percussion.render_percussion_to(buf, nsamples);
+    float gain = parameters->master * (1.0 / 8);
+    eq_l.set(parameters->bass_freq, parameters->bass_gain, parameters->treble_freq, parameters->treble_gain, sample_rate);
+    eq_r.copy_coeffs(eq_l);
+    for (int i=0; i<nsamples; i++) {
+        output[0][i] = gain*eq_l.process(buf[i][0]);
+        output[1][i] = gain*eq_r.process(buf[i][1]);
+    }
+    eq_l.sanitize();
+    eq_r.sanitize();
+}
